# File rvsao/Emvel/emrslts.x
# September 21, 1995
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1995 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.

#  Log radial velocity from emission line shift

include <imhdr.h>
include "rvsao.h"
include "emv.h"
define	SZ_MONTH	3
define	SZ_YEAR		36

procedure emrslts (specfile, mspec, image, rmode)

char	specfile[ARB]	# Name of spectrum file
int	mspec		# Spectrum number in multispec file (else 0)
pointer	image		# Header structure for spectrum
int	rmode		# Report format (1=normal,2=one-line

int	i, j
char	wch		# Line weight indicator
double	emz		# z from emission lines
double	xcz		# z from cross-correlation
double	coz		# z from cross-correlation and emission
char	instrument[8]

bool	verbose, clgetb()
double	ra, dec, ut, dj, velline[MAXREF], errline[MAXREF], eqwline[MAXREF]
double	twt, wt, exptime, alt, az, ha, lat, sha,cha,slat,clat,sdec,cdec
double	ac,dpi,tpi,rcon, rdec, rha, dindef
int	mm,dd,yyyy
char	month[SZ_YEAR]
int	icz,iczxc,iczem,icze,iczxce,iczeme

include "emv.com"
include "rvsao.com"

begin
	dindef = INDEFD
	verbose = clgetb ("verbose")
        if (!verbose) return
	call strcpy ("JanFebMarAprMayJunJulAugSepOctNovDec",month, SZ_YEAR)

	call juldate (image, ut, dj, debug)

	mm = INDEFI
	dd = INDEFI
	yyyy = INDEFI
	call imgdate (image, "DATE-OBS", mm, dd, yyyy)

	if (spvel != INDEFD) {
	    icz = idnint (spvel)
	    icze = idnint (sperr)
	    }
	else {
	    icz = 0
	    icze = 0
	    }

	if (spxvel != INDEFD) {
	    iczxc = idnint (spxvel)
	    iczxce = idnint (spxerr)
	    }
	else {
	    iczxc = 0
	    iczxce = 0
	    }

	if (spevel != INDEFD) {
	    iczem = idnint (spevel)
	    iczeme = idnint (speerr)
	    }
	else {
	    iczem = 0
	    iczeme = 0
	    }

# Set up spectrum number in specid
	if (mspec > 0) {
	    call sprintf (specid,SZ_PATHNAME,"%s[%d]")
		call pargstr (specfile)
		call pargi (mspec)
	    }
	else
	    call strcpy (specfile,specid,SZ_PATHNAME)

	if (rmode == 1) {

#	Set up heading in logfiles
	    call imgdpar (image,"RA",ra)
	    call imgdpar (image,"DEC",dec)
	    do j = 1, nlogfd {
		call fprintf(logfd[j], "\n%s Object: %s  RA: %011.2h Dec: %010.1h\n")
		    call pargstr (specid)
		    call pargstr (specname)
		    call pargd (ra)
		    call pargd (dec)
		call fprintf(logfd[j], "Observed %2d-%3.3s-%4d %011.2h = JD %.4f %s: %6.2f\n")
		    call pargi (dd)
		    call pargstr (month[(mm - 1) * SZ_MONTH + 1])
		    call pargi (yyyy)
		    call pargd (ut)
		    call pargd (dj)
		switch (svcor) {
		    case NONE:
			call pargstr ("BCV")
		    case HCV:
			call pargstr ("HCV")
		    case BCV:
			call pargstr ("BCV")
		    case FHCV:
			call pargstr ("File HCV")
		    case FBCV:
			if (specvb) 
			    call pargstr ("File BCV")
			else
			    call pargstr ("File HCV")
		    default:
		    }
		    call pargd (spechcv)
		call flush (logfd[j])

	#  Print combined velocity
		if (spvel != dindef) {
		    coz = spvel / c0
		    call fprintf(logfd[j],
		    "Combined vel = %8.2f +- %7.2f km/sec, z= %6.4f \n")
			call pargd (spvel)
			call pargd (sperr)
			call pargd (coz)
		    call flush (logfd[j])
		    }

	# Print cross-correlation velocity
		if (spxvel != dindef) {
		    xcz = spxvel / c0
		    call fprintf(logfd[j],
		    "Correlation vel = %8.2f +- %7.2f km/sec, z= %6.4f R= %6.1f \n")
			call pargd (spxvel)
			call pargd (spxerr)
			call pargd (xcz)
			call pargd (spxr)
		    call flush (logfd[j])
		    }

	# Print emission line velocity
		if (spevel != dindef) {
		    emz = spevel / c0
		    call fprintf(logfd[j],
		    "Emission vel = %8.2f +- %7.2f km/sec, z= %6.4f for %d/%d lines\n")
			call pargd (spevel)
			call pargd (speerr)
			call pargd (emz)
			call pargi (nfit)
			call pargi (nfound)
		    call flush (logfd[j])
		    }

	# Print emission line information if any were found
		if (nfound > 0) {
		    twt = 0.d0
		    do i = 1, nfound {
			wt = emparams[10,1,i]
			if (wt != 0.d0)
			    twt = twt + 1.d0 / (wt * wt)
			}
		    call fprintf(logfd[j], "\nLine  Rest lam  Obs. lam    ")
		    call fprintf(logfd[j], "Pixel     z         vel    dvel")
		    call fprintf(logfd[j], "    eqw    wt\n")
		    do i = 1, nfound {
			wtobs[i] = emparams[10,1,i]
			if (wtobs[i] > 0) {
			    if (override[i] < 0)
				wch = '-'
			    else if (override[i] > 0)
				wch = '+'
			    else
				wch = ' '
			    }
			else
			    wch = 'X'
			call fprintf(logfd[j],"%4s   %7.2f   %7.2f  %7.2f")
			    call pargstr (nmobs[1,i])
			    call pargd (wlrest[i])
			    call pargd (wlobs[i])
			    call pargd (emparams[4,1,i])
			wt = emparams[10,1,i]
			if (wt != 0.d0)
			    wt = (1.d0 / (wt * wt)) / twt
			call fprintf(logfd[j],"  %6.4f  %8.2f %7.2f %6.2f %6.3f %s\n")
			    call pargd (emparams[9,1,i])
			    call pargd ((c0*emparams[9,1,i]) + spechcv)
			    call pargd (c0*emparams[9,2,i])
			    call pargd (emparams[7,1,i])
			    call pargd (wt)
			    call pargstr (wch)
			}
		    }
		else {
		    call fprintf(logfd[j], "No emission lines found\n")
		    }
		}
	    }

# Single line report
	else if (rmode == 2) {
	    call imgspar (image,"INSTRUME",instrument,8)
	    do j = 1, nlogfd {
		call fprintf(logfd[j],"%6s %8s %16s %15.5f %5d %3d %5d %3d %5.1f %5d %3d E %2d %2d")
		    call pargstr (specid)
		    call pargstr (instrument)
		    call pargstr (IM_TITLE(image))
		    call pargd (dj)
		    call pargi (iczem)
		    call pargi (iczeme)
		    call pargi (iczxc)
		    call pargi (iczxce)
		    call pargd (spxr)
		    call pargi (icz)
		    call pargi (iczxce)
		    call pargi (nfound)
		    call pargi (nfit)
		if (nfit > 0) {
		    do i = 1, nfound {
			wtobs[i] = emparams[10,1,i]
			if (wtobs[i] > 0 && override[i] >= 0) {
			    call fprintf(logfd[j], " %s(%7.2f)")
				call pargstr (nmobs[1,i])
				call pargd (wlrest[i])
			    }
			}
		    }
		call fprintf(logfd[j], "\n")
		}
	    }

# Single line report with one velocity per possible line
	else if (rmode == 3) {
	    call imgspar (image,"INSTRUME",instrument,8)
	    call aclrd (velline,MAXREF)

#	Compute individual line velocities from shift
	    if (nfound > 0) {
		do i = 1, nfound {
		    wtobs[i] = emparams[10,1,i]
		    if (wtobs[i] > 0 && override[i] >= 0) {
			do j = 1, nref {
			    if (wlref[j] == wlrest[i])
				velline[j] = (c0 * emparams[9,1,i]) + spechcv
			    }
			}
		    }
		}

	# Print one-line report
	    do j = 1, nlogfd {
		call fprintf(logfd[j],"%6s %8s %16s %15.5f %8.2f %6.2f %8.2f %6.2f %5.1f %8.2f %6.2f E %2d %2d")
		    call pargstr (specid)
		    call pargstr (instrument)
		    call pargstr (IM_TITLE(image))
		    call pargd (dj)
		    call pargd (spevel)
		    call pargd (speerr)
		    call pargd (spxvel)
		    call pargd (spxerr)
		    call pargd (spxr)
		    call pargd (spvel)
		    call pargd (sperr)
		    call pargi (spnl)
		    call pargi (spnlf)
		do i = 1, nref {
		    call fprintf (logfd[j]," %8.2f")
			call pargd (velline[i])
		    }
		call fprintf(logfd[j], "\n")
		}
	    }

# Single line report with velocity, error, and equivalent width per line
	else if (rmode == 4) {
	    call imgspar (image,"INSTRUME",instrument,8)
	    call aclrd (velline,MAXREF)

#	Compute individual line velocities from shift
	    if (nfound > 0) {
		do i = 1, nfound {
		    wtobs[i] = emparams[10,1,i]
		    if (wtobs[i] > 0 && override[i] >= 0) {
			do j = 1, nref {
			    if (wlref[j] == wlrest[i]) {
				velline[j] = (c0 * emparams[9,1,i]) + spechcv
				errline[j] = c0 * emparams[9,2,i]
				eqwline[j] = emparams[7,1,i]
				}
			    }
			}
		    }
		}

	# Get sky position
	    ra = 99.d0
	    call imgdpar (image,"RA",ra)
	    dec = 99.d0
	    call imgdpar (image,"DEC",dec)
	    ha = 99.d0
	    call imgdpar (image,"HA",ha)
	    lat = 99.d0
	    call imgdpar (image,"SITELAT",lat)

	    if (debug) {
		call printf ("EMRSLTS: RA: %011.2h Dec: %010.1h HA: %011.2h Lat: %010.1h\n")
		    call pargd (ra)
		    call pargd (dec)
		    call pargd (ha)
		    call pargd (lat)
		}

	# Compute altitude and azimuth
	    if (ha < 12.d0 && ha > -12.d0 && dec <= 90.d0 && dec >= -90.d0 &&
 		lat >= -90.d0 && lat <= 90.d0) {
		dpi = 3.1415926535897932d0
		tpi = 6.283185307179588d0
		rcon = 1.74532925199433d-2
		rha = ha * 15.d0 * rcon
		if (rha == 0)
		    rha = 1.d-5
		sha = sin (rha)
		cha = cos (rha)
		rdec = dec * rcon
		sdec = sin (rdec)
		cdec = cos (rdec)
		slat = sin (lat)
		clat = cos (lat)
		ac = (clat * sdec) - (slat * cdec * cha)
		az = atan (-cdec * sha / abs (ac))
		if (ac < 0) az = dpi - az
		if (az < 0) az = tpi + az
		ac = -cdec * sha / sin (az)
		alt = atan ((slat * sdec) + (clat * cdec * cha) / abs (ac))
		if (ac < 0) alt = dpi - alt
		az = az / rcon
		alt = alt / rcon
		}
	    else {
		alt = 0.d0
		az = 0.d0
		}

	    exptime = 0.d0
	    call imgdpar (image,"EXPTIME",exptime)

	# Print one-line report
	    do j = 1, nlogfd {
		call fprintf(logfd[j],"%6s %8s %16s")
		    call pargstr (specid)
		    call pargstr (instrument)
		    call pargstr (IM_TITLE(image))
		call fprintf(logfd[j]," %011.2h %010.1h %5.1f %5.1f")
		    call pargd (ra)
		    call pargd (dec)
		    call pargd (az)
		    call pargd (alt)
		call fprintf(logfd[j]," %15.5f %6.1f")
		    call pargd (dj)
		    call pargd (exptime)
		call fprintf(logfd[j]," %8.2f %6.2f %8.2f %6.2f %5.1f")
		    call pargd (spevel)
		    call pargd (speerr)
		    call pargd (spxvel)
		    call pargd (spxerr)
		    call pargd (spxr)
		call fprintf(logfd[j]," %8.2f %6.2f %2d %2d")
		    call pargd (spvel)
		    call pargd (sperr)
		    call pargi (spnl)
		    call pargi (spnlf)
		do i = 1, nref {
		    call fprintf (logfd[j]," %8.2f %6.2f %6.2f")
			call pargd (velline[i])
			call pargd (errline[i])
			call pargd (eqwline[i])
		    }
		call fprintf(logfd[j], "\n")
		}
	    }
end
# Aug 12 1992	Correctly print z, not 1+z for fit velocities
# Dec  2 1993	Print spectrum number if multispec file

# Apr  8 1993	Set speed of light locally

# Apr 13 1994	Remove unused variables k0, k1 and rlim
# Apr 22 1994	Remove unused variable gwidth
# Apr 25 1994	Declare month as char
# May 17 1994	Add second option for single-line output format
# Jun 15 1994	Add third output option for per-line velocities
# Aug  3 1994	Change common and header from fquot to rvsao
# Aug 10 1994	Fix ut time
# Aug 17 1994	Add weight per line for mode 1 report
# Aug 22 1994	Don't add weight if zero

# Feb  9 1995	Give 2 decimal places for all velocities
# Feb 15 1995	Add heliocentric vel. correction to individual line velocities
# Apr  3 1995	Add fourth output option for single-line per-line velocities
# Apr  5 1995	Add sky coordinates, alt/az and exposure time to 4th option
# Apr 10 1995	Fix alt/az
# May 15 1995	Change all sz_fname to sz_pathname, which is longer
# Jul 13 1995	Get Julian Date from JULDATE instead of JULDAY
# Jul 13 1995	Get UT from JULDATE
# Jul 19 1995	Handle INDEF values for SPVEL and SPXVEL
# Sep 21 1995	Handle INDEF value for SPEVEL
