# File rvsao/Fquot/fqfit.x
# September 21, 1995
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# After Gerard Kriss (June 1987), Johns Hopkins University
 
# FQFIT is the heart of the FQUOT task.  It fits each template spectrum
# in tempspec to the given galaxy spectrum using the FORTRAN routines
# originally written by Paul Schechter.  Data is shared in common
# defined in "rvsao.com"
 
include	"rvsao.h"
 
procedure fqfit (spectrum, tempspec, wltemp, chisq, itemp)
 
real	spectrum[ARB]	# Spectrum of object to be fit
real	tempspec[ARB]	# Template spectrum
real	wltemp[ARB]	# Wavelengths for object and template spectra
real	chisq		# RETURNED reduced Chi-square for the fit
int	itemp		# template sequence number
 
int	log2, i, lup, nhalf
int	ishift
int	nparam
real	kmin
double	specavg, tempavg
double	t1, t2
real	x[MAXPTS2]
real	ye[MAXPTS2], qr, qi
real	specr, speci, tempr, tempi
real	xvec[MAXPTS]
double	spectot, temptot, dnpts
complex	specft[MAXPTS2], tmpft[MAXPTS2], quot[MAXPTS2]
char	title[SZ_LINE],nothing[2]
pointer	work
#complex gamn
int	nsmooth
#int	iwk[20]
 
real	chiper(), alog()
 
include	"rvsao.com"
include	"fquot.com"
 
begin
	nsmooth = 0
	nothing[1] = EOS
	nothing[2] = EOS

# Zero the FFT arrays
 
	call aclrx (specft,MAXPTS2)
	call aclrx (tmpft,MAXPTS2)

# Allocate work vector
	if (work != NULL)
	    call mfree (work, TY_REAL)
	call malloc (work, npts, TY_REAL)

# Set initial values for parameters
	log2 = alog (real (npts)) / LN2 + 0.001
	nhalf = npts / 2
	ishift = alog10 (1. + z0 / C) / dlogw
	z[1] = -ishift
	z[2] = alog (sig0 / deltav)
	z[3] = gam0

	dnpts = double (npts)

# Subtract continuum and chop emission lines from spectrum using IRAF ICFIT
	call icsubcon (npts,spectrum,wltemp,specname,1,Memr[work])

# Subtract continuum and chop emission lines from template using IRAF ICFIT
	call icsubcon (npts,tempspec,wltemp,tempname[1,itemp],2,Memr[work])

	call mfree (work, TY_REAL)

# Apodize ends of spectrum
	call apodize (npts,spectrum,wltemp,han,"Spectrum apodized: ",
		      specname)

# Apodize ends of template
	call apodize (npts,tempspec,wltemp,han,"Template apodized: ",
		      tempname[1,itemp])

# Fourier transform spectrum
	call rcvec (npts,spectrum,specft)
	call fourm (specft,log2,1)
	if (pltfft) {
	    call strcpy ("Spectrum fft: ", title, SZ_LINE)
	    call strcat (specname, title, SZ_LINE)
	    do i = 1, npts {
		xvec[i] = i
		}
	    call plotspec (npts, specft, title, xvec,nothing,nsmooth)
	    }

# Fourier transform template
	call rcvec (npts,tempspec,tmpft)
	call fourm (tmpft,log2,1)
	if (pltfft) {
	    call strcpy ("Template fft: ", title, SZ_LINE)
	    call strcat (tempname[1,itemp], title, SZ_LINE)
	    call plotspec (npts, tmpft, title, xvec,nothing,nsmooth)
	    }
 
# Compute preliminary error array
	spectot = 0.d0
	temptot = 0.d0
	do i = 1, npts {
	    temptot = temptot + tempspec[i]
	    spectot = spectot + spectrum[i]
	    }
	specavg = spectot / dnpts
	tempavg = temptot / dnpts
	if (debug) {
	    call printf("FQFIT: specavg = %f, tempavg = %f\n")
		call pargd (specavg)
		call pargd (tempavg)
	    }

	if (debug)
	    call printf("FQFIT: Computing preliminary error array.\n")
	if (specavg <= 0)
	    t1 = cpf * dnpts / -specavg
	else
	    t1 = cpf * dnpts / specavg
	if (tempavg <= 0)
	    t2 = cpf * dnpts / -tempavg
	else
	    t2 = cpf * dnpts / tempavg
	do i = 1, nhalf {
	    specr = real (specft[i])
	    speci = aimag (specft[i])
	    tempr = real (tmpft[i])
	    tempi = aimag (tmpft[i])
	    ye[i] = (t1 / (specr*specr + speci*speci)) +
		    (t2 / (tempr*tempr + tempi*tempi))
	    }

# Determine high frequency cutoff
	call runsum (ye, npts, lup, nrun, chi0)
	if (lup < lo + 20)
	    lo = max0 (1, lup-20)
 
# Compute the Fourier quotient
	kmin = TWOPI / npts
	do i = 1, lup {
	    x[i] = kmin * (i - 1)
	    quot[i] = specft[i] / tmpft[i]
	    qr = real (quot[i])
	    qi = aimag (quot[i])
	    ye[i] = sqrt (ye[i] * (qr*qr + qi*qi) )
	    if (debug) {
#		call printf ("FQFIT: %d x= %f, quot= %f + %fi, ye= %f\n")
#		    call pargi (i)
#		    call pargr (x[i])
#		    call pargr (qr)
#		    call pargr (qi)
#		    call pargr (ye[i])
		}
	    }
	nparam = 3
	if (pltfft) {
	    call strcpy ("Fourier quotient: ", title, SZ_LINE)
	    call strcat (specname, title, SZ_LINE)
	    call strcat (specname, " / ", SZ_LINE)
	    call strcat (tempname[1,itemp], title, SZ_LINE)
	    call plotspec (lup, ye, title, xvec,nothing,nsmooth)
	    }

# Fit a Gaussian to the Fourier quotient
	chisq = chiper (x, quot, ye, unc, lo, lup, z, ze, nparam, niter)
 
# Show the fit if requested
	if (pltcor)
	    call plotfit (x, quot, specname, lo, lup, z)
 
end
# Nov 20 1991	Free work vector before allocating it

# Feb 19 1992	Pass wavelength vector for diagnostic plots
# Jun 16 1992	Drop old polynomial continuum fit
# Oct  9 1992	Compute SPECAVG and TEMPAVG
# Dec  3 1992	Major debugging

# Jan 20 1993	Set SPECAVG and TEMPAVG to m

# Apr 22 1994	Drop unused variable logscale
# May  9 1994	Add argument to PLOTSPEC; optionally plot quotient
# Aug  3 1994	Change common and header from fquot to rvsao
# Nov 16 1994	Define null string before passing it

# Mar 15 1995	Pass object and template names to ICSUBCON
# Sep 21 1995	Move FQUOT parameters to fquot.com
