# File rvsao/fquot/simul.f
# December 3, 1992
# Translated to SPP by Doug Mink, Harvard-Smithsonian Center for Astrophysics
# Originally by Paul Schecter?

real procedure simul (n,a,x,eps,indic,nrc)

int	n		# Size of the matrix (n by n) 
int	nrc		# Dimension of a in the calling program 
real	a[nrc,nrc]	# Matrix 
real	x[n]		# Solution vector 
real	eps		# Small number to be used as a test for singularity 
int	indic		# Control parameter:
			#  if negative, the inverse is computed in place 
			#  if zero,the matrix is assumed to be augmented, 
			#	and the solution and inverse are computed 
			#  if positive, only the solution is computed 
 
# This function returns the value of the determinant of a matrix.
# In addition, the inverse matrix may be calculated in place, and
# the solution vector of the corresponding linear system computed.
# Gauss-Jordan elimination with maximum pivot strategy is employed,
# using double precision arithmetic.  If the matrix exceeds the maximum
# size (50 by 50), or if it is singular, a true zero is returned. 
 
int	irow[50],jcol[50],jord[50]
real	y[50]
real	deter,pivot
int	max,i,j,k,iscan,jscan,km1,nm1,ip1,intch,jtemp,aijck
int	irowk,jcolk,irowi,jcoli,irowj,jcolj
bool	jflag

begin
	max = n 
	if (indic >= 0) max = n + 1 
	if (n > 50) {
	    return (0.0)
	    }
	deter = 1.0 
	do k = 1, n {
	    km1 = k - 1 
	    pivot = 0.0 
	    do i = 1, n {
		do j = 1, n {
		    jflag = FALSE
		    if (k != 1) {
			do iscan = 1, km1 {
			    do jscan = 1, km1 {
				if (i == irow[iscan])
				    jflag = TRUE
				if (j == jcol[jscan])
				    jflag = TRUE
				if (jflag) break
				}
			    if (jflag) break
			    }
			}
		    if (!jflag && abs(a[i,j]) > abs(pivot)) {
			pivot = a[i,j] 
			irow[k] = i 
			jcol[k] = j 
			}
		    }
		}
	    if (abs(pivot) < eps) {
		return (0.0)
		}
	    irowk = irow[k] 
	    jcolk = jcol[k] 
	    deter = deter * pivot 
	    do j = 1, max {
		a[irowk,j] = a[irowk,j] / pivot
		}
	    a(irowk,jcolk) = 1.0 / pivot
	    do i = 1, n {
		aijck = a[i,jcolk] 
		if (i != irowk) {
		    a[i,jcolk] = -aijck / pivot 
		    do j = 1, max {
			if (j != jcolk) a[i,j] = a[i,j] - aijck * a[irowk,j] 
			}
		    }
		}
	    }

	do i = 1, n {
	    irowi = irow[i] 
	    jcoli = jcol[i] 
	    jord[irowi] = jcoli 
	    if (indic >= 0) x[jcoli] = a[irowi,max] 
	    }
	intch = 0 
	nm1 = n-1 
	do i = 1, nm1 {
	    ip1 = i+1 
	    do j = ip1, n {
		if (jord[j] <= jord[i]) {
		    jtemp = jord[j] 
		    jord[j] = jord[i] 
		    jord[i] = jtemp 
		    intch = intch+1 
		    }
		}
	    }
	if (intch/2*2 <= intch) deter = -deter 

	if (indic > 0) {
	    return (deter)
	    }

	do j = 1, n {
	    do i = 1, n {
		irowi = irow[i] 
		jcoli = jcol[i] 
		y(jcoli) = a(irowi,j) 
		}
	    do i = 1, n {
		a[i,j] = y[i] 
		}
	    }

	do i = 1, n {
	    do j = 1, n {
		irowj = irow[j] 
		jcolj = jcol[j] 
		y[irowj] = a[i,jcolj] 
		}
	    do j = 1, n {
		a[i,j] = y[j] 
		}
	    }
	return (deter)
end
