# File rvsao/Util/compbcv.x
# September 25, 1995
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1995 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.

include	<imhdr.h>

real procedure compbcv (im,ihcv,debug)

pointer	im		# image structure for spectrum
int	ihcv		# If =1, return barycentric velocity, else heliocentric
bool	debug		# If true, display diagnostics

double	ra, dec, epoch, dindef
double	dj, dlong, dlat, dalt, ut
double	dhcv, dbcv, dgcv
real	rbcv, rhcv

begin
	dindef = INDEFD

# Julian Date of midtime of observation
	call juldate (im, ut, dj, debug)

# Direction of observed object
	ra = dindef
 	call imgdpar (im,"RA",ra)
	dec = dindef
	call imgdpar (im,"DEC",dec)
	epoch = 1950.00
	call imgdpar (im,"EPOCH",epoch)

# Position of observatory
	dlong = dindef
 	call imgdpar (im,"SITELONG",dlong)
	dlat = dindef
 	call imgdpar (im,"SITELAT",dlat)
	dalt = dindef
 	call imgdpar (im,"SITEELEV",dalt)

# Barycentric velocity correction
	dhcv = 0.d0
	dbcv = 0.d0
	dgcv = 0.d0
	if (ra != dindef && dec != dindef && dj != dindef &&
	    dlong != dindef && dlat != dindef && dalt != dindef) {
	    call bcv (dj, dlong,dlat,dalt, ra,dec,epoch, dhcv, dbcv, dgcv)
#	    call printf ("bcv is %f\n")
#	    call pargr (rbcv)
	    }
	if (debug) {
	    call printf ("BCVCORR: RA = %.3h  Dec = %.2h\n")
		call pargd (ra)
		call pargd (dec)
	    call printf ("BCVCORR: Obs lat = %.3h  long = %.3h  alt = %.2f\n")
		call pargd (dlat)
		call pargd (dlong)
		call pargd (dalt)
	    }
	rhcv = dhcv + dgcv
	rbcv = dbcv + dgcv

	if (debug) {
	    call printf ("COMPBCV: %.4f: hcv= %.3f, bcv = %.3f, gcv = %.3f\n")
		call pargd (dj)
		call pargd (dhcv)
		call pargd (dbcv)
		call pargd (dgcv)
	    call printf ("COMPBCV:       helio.corr.= %.3f, bary.corr. = %.3f\n")
		call pargr (rhcv)
		call pargr (rbcv)
	    }
	if (ihcv == 1)
	    return (rbcv)       
	else
	    return (rhcv)       
end
# Apr 13 1994	Clean up code after running ftnchek
# Aug  3 1994	Change common and header from fquot to rvsao

# Jun 27 1995	Initialize all header parameters to indef
# Jul 13 1995	Compute Julian Date in a separate subroutine
# Jul 17 1995	Pass geocentric back from bcv, too.
# Sep 22 1995	Initialize velocity correction to zero
