# File rvsao/Util/getsky.x
# July 8, 1996
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1994 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
 
#  GETSKY opens the image specified by specfile and returns pointers
#  to the sky data and the image descriptor.
 
#  Notes:	Shares data in "rvsao.com"
 
###########################################################################
 
include	"rvsao.h"
include "shdr.h"
include <imhdr.h>
 
procedure getsky (specfile, mspec, spectrum, specim, skysh)
 
char	specfile[ARB]	# Data file name
int	mspec		# Number of spectrum to read from multispec file
pointer	spectrum	# Spectrum data (returned)
pointer	specim		# Image header structure (returned)
pointer	skysh		# Spectrum header structure (returned)
 
int	npix
int	nline
 
real	compbcv()
double	dlog10()
 
include	"rvsao.com"
include	"getim.com"
 
begin
	nline = mspec
	if (debug) {
	    call printf ("GETSKY: %s %d line %d\n")
		call pargstr (specfile)
		call pargi (mspec)
		call pargi (nline)
	    }
	call getimage (specfile,nline,spectrum,specim,skysh,npix,
		       skyname,READ_ONLY, debug)
	if (specim == ERR) {
	    call printf ("GETSKY:  Error reading %s[%d]\n")
		call pargstr (specfile)
		call pargi (nline)
	    return
	    }
 
# redefine common variables
	specpix = npix
	specdc = DC(skysh)

# Calculate heliocentric velocity, or use one from file
	switch (svcor) {
	    case NONE:
		spechcv = 0.
	    case HCV:
		spechcv = compbcv (specim,0,debug)
	    case BCV:
		spechcv = compbcv (specim,1,debug)
	    case FBCV:
		specvb = vbc
		if (specvb)
		    spechcv = fbcv
		else
		    spechcv = fhcv
	    case FHCV:
		spechcv = fhcv
		specvb = vbc
	    }

# Compute log wavelength image of spectrum
	if (specdc == 0) {
	    spstrt = dlog10(w0)
	    spfnsh = dlog10(w0 + wpc * dble (npix-1))
	    }
	else if (specdc == 1) {
	    spstrt = w0
	    spfnsh = w0 + wpc * dble (npix-1)
	    }

end

# Apr 22 1992	Modify to handle multispec spectra

# Jun  4 1993	Modify to handle MWCS spectra
# Jul  7 1993	Add spectrum header to getimage arguments

# Apr 12 1994	Keep sky name separate from spectrum name; fix debug code
# Apr 12 1994	Return MWCS header pointer
# Jun 15 1994	Set SPECDC from DC(SKYSH), not DCFLAG
# Jun 23 1994	Keep mWCS pointer in SHDR structure
# Aug  3 1994	Change common and header from fquot to rvsao

# Jul 13 1995	Add DEBUG to COMPBCV calls

# Jul  8 1996	Pass debugging flag to getimage
