# File rvsao/Xcor/xcplot.x
# September 25, 1995
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1995 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.

#  XCPLOT uses IRAF plot routines to plot a spectrum and its cross-correlation.
#  The spectrum can also be plotted with labelled absorption lines.

###########################################################################

include <gset.h>
include <gio.h>
include <pkg/gtools.h>
include <ttyset.h>
include <fset.h>
include "rvsao.h"
include "emv.h"

procedure xcplot (title, wlspec,spectrum,npix,velaxs,velcor,nvel
		  specfile,mspec,specim,newfit)

char	title[ARB]	# Tile for summary display
real	wlspec[ARB]	# Wavelengths for object spectrum
real	spectrum[ARB]	# Object spectrum
int	npix		# Number of pixels in object spectrum
real	velaxs[ARB]	# Velocities for cross-correlation
real	velcor[ARB]	# Cross-correlation
int	nvel		# Number of points in cross-correlation
char	specfile[ARB]	# Spectrum file name
int	mspec		# Number of spectrum to read from multispec file
pointer	specim		# Spectrum image header structure
bool	newfit		# Rerun correlation if true

double	strtwav		# Wavelength of first correlation pixel
double	finwav		# Wavelength of last correlation pixel
char    command[SZ_FNAME]
double	dtemp
int	itemp, i, ip
char	linebuf[SZ_LINE]
int     wc, key
real	wx, wy
pointer gt, gfd, gfp
char    device[SZ_FNAME] # Display on which to plot data
char    plotter[SZ_FNAME] # Device on which to make hardcopy
bool    cursor          # true if waiting for cursor after plotting
bool    hardcopy        # true to make automatic hard copy and plot
bool	displot
int	dispmode
pointer	sp
pointer	openplot()
int	ipix1, ipix2, ix, npix0
double	wl
bool	zfirst

pointer	smspec		# Smoothed object spectrum
int	getline()
int	gt_gcur()
pointer	gt_init(), gopen()
bool	clgetb()
int	clgeti()
int	ctoi()
double	ctod()
define	endplot_	90

include "rvsao.com"
include "results.com"
include "emv.com"

begin

# Smooth spectrum for plotting
	call smark (sp)
	call salloc (smspec, npix, TY_REAL)
	call amovr (spectrum,Memr[smspec],npix)
	call smooth (Memr[smspec],npix,nsmooth)
	strtwav = twl1[itmax]
	finwav = twl2[itmax]

	dispmode = clgeti ("dispmode")
	dispabs = TRUE
	zfirst = TRUE
	if (spnl > 0)
	    dispem = TRUE
	else {
	    dispem = FALSE
	    do i = 1, ntemp {
		if (tschop[i] == 0 && czr[i] > 4.0)
		    dispem = TRUE
		}
	    }
	vplot = VCORREL

# Print hard copy, if requested

	hardcopy = clgetb ("hardcopy")
	if (hardcopy) {

	# Open plotter
	    call clgstr ("plotter",plotter,SZ_FNAME)
	    gfp = openplot (plotter)

	    if (dispmode == 2) {
		call emeplot(gfp,specfile,specim,mspec,Memr[smspec],wlspec,npix)
		}

	# Plot spectrum, cross-correlation, and template information
	    else {
		call xcsplot (gfp,npix,wlspec,Memr[smspec],strtwav,finwav)
		call xcorplot (gfp,nvel,velaxs,velcor,title)
		call xciplot (gfp,specfile,specim,mspec,strtwav,finwav)
		}

	# Close plotter
	    call closeplot (gfp)
	    }

#  If not displaying data, return
	displot = clgetb ("displot")
	if (!displot) {
	    call sfree (sp)
	    return
	    }

# Plot spectrum with absorption and emission lines labelled
	if (dispmode == 2) {
	    xfit = FALSE
	    call emplot (specfile,specim,mspec,npix,spectrum,
			 Memr[smspec],wlspec,strtwav,finwav)
	    if (!plotcorr) {
		if (xfit) {
		    call printf ("Rerun cross-correlation\n")
		    newfit = TRUE
		    }
		call sfree (sp)
		return
		}
	    call printf ("Plotting correlation with summary\n")
	    }

#  Open display
	call clgstr ("device",device,SZ_FNAME)
	gfd = openplot (device)

#  Plot spectrum, cross-correlation and template information
	 call xcsplot (gfd,npix,wlspec,Memr[smspec],strtwav,finwav)
	 call xcorplot (gfd,nvel,velaxs,velcor,title)
	 call xciplot (gfd,specfile,specim,mspec,strtwav,finwav)

#  If not using cursor, return
	cursor = clgetb ("curmode")
	if (!cursor) {
	    call closeplot (gfd)
	    call sfree (sp)
	    return
	    }

#  Accept cursor commands
	newfit = FALSE
	gt = gt_init()
	ipix1 = 1
	ipix2 = npix
	npix0 = npix
	while (gt_gcur ("cursor", wx, wy, wc, key, command, SZ_FNAME) != EOF) {
	    wl = wx
	    ix = ipix1
	    do i = ipix1, ipix2 {
		if (wl >= wlspec[i] && wx < wlspec[i+1]) {
		    if (wl - wlspec[i] > 0.5)
			ix = i
		    else
			ix = i + 1
		    break
		    }
		}

	    switch (key) {

	    # Toggle debug flag
		case 'd':
		    call gclose (gfd)
		    if (debug) {
			call printf ("debugging off\n")
			debug = FALSE
			}
		    else {
			debug = TRUE
			call printf ("debugging on\n")
			}
		    gfd = gopen (device, APPEND, STDGRAPH)

	    # Display spectrum so that it can be edited
		case 'e':
		    call gclose (gfd)
		    call printf ("edit spectrum\n")
		    call plotspec (npix,spectrum,title,wlspec,"Wavelength",nsmooth)
		    gfd = gopen (device, APPEND, STDGRAPH)
		    newfit = TRUE
		    break

#	Set number of times to smooth spectrum
		case 'g':
		    call gclose (gfd)
		    call printf("Number of times to smooth spectrum (%d): ")
			call pargi (nsmooth)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 1) {
			ip = 1
			i = ctoi (linebuf,ip,itemp)
			nsmooth = itemp
			call amovr (spectrum,Memr[smspec],npix)
			call smooth (Memr[smspec],npix,nsmooth)
			}
		    gfd = gopen (device, APPEND, STDGRAPH)

#	Select different velocity for final redshift

	    # Repeat correlation plot to select peak using cursor
		case 'p':
		    call gclose (gfd)
		    call printf ("select correlation peak(s)\n")
		    pltcor  = TRUE
		    newfit = TRUE
		    gfd = gopen (device, APPEND, STDGRAPH)
		    break

	    # Plot spectrum with absorption lines marked
		case 'l':
		    call gclose (gfd)
		    call printf ("Plotting absorption lines with summary\n")
		    xfit = FALSE
		    call emplot (specfile,specim,mspec,npix,spectrum,
				 Memr[smspec],wlspec,strtwav,finwav)
		    if (!plotcorr) {
			if (xfit) {
			    newfit = TRUE
			    call printf ("Rerun cross-correlation\n")
			    }
			call gt_free(gt)
			call sfree (sp)
			return
			}
		    call printf ("Plotting correlation with summary\n")
		    gfd = openplot (device)
		    call xcsplot (gfd,npix,wlspec,Memr[smspec],strtwav,finwav)
		    call xcorplot (gfd,nvel,velaxs,velcor,title)
		    call xciplot (gfd,specfile,specim,mspec,strtwav,finwav)
	      	    call gflush (gfd)

	    # Change peak fitting parameters
		case 'c':
		    call gclose (gfd)
	    	    call printf ("Correlation peak fitting parameters\n")
		    call printf ("Peak-fitting mode (1-parabola 2-quartic 3-cosx/1+x^2) (%d) = ")
			call pargi (pkmode0)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 1) {
			ip = 1
			i = ctoi (linebuf,ip,itemp)
			pkmode0 = itemp
			newfit = TRUE
			}

		    call printf("Fraction of peak or number of points for peak fitting (%7.3f) = ")
			call pargd (pkfrac)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 1) {
			ip = 1
			i = ctod (linebuf,ip,dtemp)
			pkfrac = dtemp
			newfit = TRUE
			}
		    gfd = gopen (device, APPEND, STDGRAPH)
		    if (newfit) break

		case 'f':
		    call gclose (gfd)
		    call printf ("Rerunning correlations\n")
		    newfit = TRUE
		    gfd = gopen (device, APPEND, STDGRAPH)
		    break

	    # Change filter parameters
		case 't':
		    call gclose (gfd)
		    call printf ("Filter parameters for transform (%d %d %d %d) = ")
			call pargi (lo)
			call pargi (toplo)
			call pargi (nrun)
			call pargi (topnrn)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 1) {
			ip = 1
			i = ctoi (linebuf,ip,itemp)
			lo = itemp
			if (i > 0) {
			    i = ctoi (linebuf,ip,itemp)
			    toplo = itemp
			    }
			if (i > 0) {
			    i = ctoi (linebuf,ip,itemp)
			    nrun = itemp
			    }
			if (i > 0) {
			    i = ctoi (linebuf,ip,itemp)
			    topnrn = itemp
			    }
			newfit = TRUE
			}
		    gfd = gopen (device, APPEND, STDGRAPH)
		    if (newfit) break

	    # Change velocity range over which to search for peak
		case 'v':
		    call gclose (gfd)
		    call printf ("Minimum allowable velocity in km/sec (%8.1f) = ")
			call pargd (minvel)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 1) {
			ip = 1
			i = ctod (linebuf,ip,dtemp)
			minvel = dtemp
			newfit = TRUE
			}
		    call printf ("Maximum allowable velocity in km/sec (%8.1f) = ")
			call pargd (maxvel)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 1) {
			ip = 1
			i = ctod (linebuf,ip,dtemp)
			maxvel = dtemp
			newfit = TRUE
			}
		    newfit = TRUE
		    gfd = gopen (device, APPEND, STDGRAPH)
		    break

#	Set quality flag in header
		case 'y':
		    spvqual = 4
		    call gclose (gfd)
		    call printf ("Setting quality flag to 4=good\n")
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)
		    qplot = TRUE

		case 'n':
		    spvqual = 3
		    call gclose (gfd)
		    call printf ("Setting quality flag to 3=bad\n")
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)
		    qplot = TRUE

		case 'j':
		    spvqual = 1
		    call gclose (gfd)
		    call printf ("Setting quality flag to 1=questionable\n")
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)
		    qplot = TRUE

	    # Make a hard copy of the correlation summary display
		case '@':
		    call gclose (gfd)
		    call clgstr ("plotter",plotter,SZ_FNAME)
		    call printf ("Making hard copy on %s\n")
			call pargstr (plotter)
		    gfp = openplot (plotter)
		    call xcsplot (gfp,npix,wlspec,Memr[smspec],strtwav,finwav)
		    call xcorplot (gfp,nvel,velaxs,velcor,title)
		    call xciplot (gfp,specfile,specim,mspec,strtwav,finwav)
	      	    call closeplot (gfp)
		    gfd = gopen (device, APPEND, STDGRAPH)

	    # Replot correlation summary
		case 'r':
		    call gclose (gfd)
		    call printf ("Replot XCSAO summary\n")
		    gfd = openplot (device)
		    call xcsplot (gfd,npix,wlspec,Memr[smspec],strtwav,finwav)
		    call xcorplot (gfd,nvel,velaxs,velcor,title)
		    call xciplot (gfd,specfile,specim,mspec,strtwav,finwav)
	      	    call gflush (gfd)

	    # Exit without updating the image header
		case 'x':
		    qplot = NO
		    break

	    # List available cursor commands
		case ' ','h','?':
		    call gclose (gfd)
		    call printf("XCSAO commands:\n")
		    call printf("e	edit spectrum\n")
		    call printf("c	change correlation peak fit parameters\n")
		    call printf("d	toggle debug flag\n")
		    call printf("f	rerun cross-correlation\n")
		    call printf("j	conditional velocity\n")
		    call printf("l	plot spectrum with absorption lines\n")
		    call printf("n	disapprove velocity\n")
		    call printf("p	refit correlation peak interactively\n")
		    call printf("q	exit from this spectrum\n")
		    call printf("r	replot summary\n")
		    call printf("t	change transform filter parameters\n")
		    call printf("v	change velocity limits for result\n")
		    call printf("x	Exit without updating header\n")
		    call printf("y	approve velocity\n")
		    call printf("@	make hard copy\n")
		    gfd = gopen (device, APPEND, STDGRAPH)

		default:
		    break
		}
	    }

#  Close graphics window
endplot_
	call gt_free(gt)
	call closeplot (gfd)
	call sfree (sp)
	return
end

# Jun  2 1992	Fix peak selection mode bug--reopen gfd
# Nov 30 1992	Smooth spectrum here so it can be resmoothed by EMPLOT

# Jan 14 1993	Fix misspellings
# Dec  3 1993	Pass mspec to EMPLOT

# Mar 16 1994	Add option of editing spectrum
# Apr 15 1994	Add wavelength limits to EMPLOT call
# Apr 22 1994	Drop image structure from EMPLOT argument list
# May  3 1994	Add smoothed vector argument to PLOTSPEC call
# May  5 1994	Add correct smoothed vector argument to SPECPLOT call
# May  9 1994	Drop smoothed vector argument from SPECPLOT call
# May  9 1994	Add number of times to smooth spectrum to PLOTSPEC call
# May 23 1994	Locally allocate and free smoothed spectrum
# Aug  3 1994	Change common and header from fquot to rvsao
# Aug  5 1994	Drop nvel as argument to INFOPLOT
# Aug 10 1994	Plot spectrum over wavelength overlap with best template
# Aug 17 1994	Change names of SPECPLOT and INFOPLOT to XCSPLOT and XCIPLOT
# Dec  7 1994	Add multiple display modes; add x command

# Feb 27 1995	Fix hard copy command
# Mar 22 1995	Add options to change fitting parameters
# Aug 18 1995	Set MINVEL and MAXVEL in common, not parameter file
# Aug 25 1995	Add qplot velocity accepatance flags; remove unused variables
# Aug 25 1995	Turn on emission line marking if R>4 for emission template
# Sep 19 1995	Add g to change spectrum smoothing for display
# Sep 25 1995	Use EMPLOT for interaction, not EMEPLOT
