# File rvsao/Xcsao/xcsplot.x
# June 19, 1995
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# After Stephen Levine and Jon Morse

# Copyright(c) 1995 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.

include	<gset.h>
include	<gio.h>
include	"rvsao.h"
 
#  Plot spectrum on XCSAO summary page
 
procedure xcsplot (gp, npoints, xpts, ypts, minwav, maxwav)
 
pointer gp		# Graphics structure pointer
int 	npoints		# Number of points in spectrum
real	xpts[ARB]	# Array of wavelengths to plot
real	ypts[ARB]	# Array of intensities to plot
double	minwav		# Minimum wavelength for cross-correlation
double	maxwav		# Maximum wavelength for cross-correlation

int	npw		# Number of points in spectrum to plot
int	ipw		# First wavelength point to plot
int	i
pointer	w

include "rvsao.com"
include "results.com"

begin

	call gseti (gp, G_WCS, 1)

# Set font and size of tick labels
	call gseti (gp,G_TXQUALITY,GT_HIGH)
	call gsetr (gp,G_TICKLABELSIZE,0.65)

# Find points within wavelength limits
	ipw = 1
	npw = npoints
	do i = 1, npoints {
	    if (xpts[i] < minwav)
		ipw = i + 1
	    if (xpts[i] > maxwav) {
		npw = i - 1
		break
		}
	    }
	npw = npw - ipw + 1
 
# Set scale for plot of data array
	call gswind (gp, INDEF, INDEF, 0., INDEF)

# Set the window x (wavelength) limits.
	w = GP_WCSPTR (gp, GP_WCS(gp))
	WCS_WX1(w) = minwav
	WCS_WX2(w) = maxwav

	GP_WCSSTATE(gp) = MODIFIED
	call gpl_reset()

	if (npw > 0)
	    call rvscale (gp, ypts[ipw], npw, 3)

# Set viewport to half page         
	call gsview (gp, 0.06, 0.66, 0.60, 0.90)

# Plot axes
       	call glabax (gp, "", "Wavelength", "")

# Plot spectrum
	if (npw > 0)
	    call gpline (gp, xpts[ipw], ypts[ipw], npw)
	call gflush (gp)
end
# May 10 1995	Flush graphics buffer
# Jun 19 1995	If no points inside wavelength limits, do not try to plot

# Jul  8 1996	Autoscale Y such that 0 is always included
